@@ -6,7 +6,7 @@ Kafka::Connection - Object interface to connect to a kafka cluster.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Connection> version 0.8010 .
+This documentation refers to C<Kafka::Connection> version 0.8011 .
 
 =cut
 
@@ -20,7 +20,7 @@ use warnings;
 
 our $DEBUG = 0;
 
-our $VERSION = '0.8010';
+our $VERSION = '0.8011';
 
 use Exporter qw(
     import
@@ -79,6 +79,7 @@ use Kafka qw(
     $ERROR_MESSAGE_SIZE_TOO_LARGE
     $ERROR_STALE_CONTROLLER_EPOCH_CODE
     $ERROR_OFFSET_METADATA_TOO_LARGE_CODE
+    $ERROR_CONSUMER_COORDINATOR_NOT_AVAILABLE_CODE
 
     $ERROR_CANNOT_BIND
     $ERROR_CANNOT_GET_METADATA
@@ -143,15 +144,17 @@ use Kafka::Protocol qw(
     try {
         $connection = Kafka::Connection->new( host => 'localhost' );
     } catch {
-        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
-            warn $_->message, "\n", $_->trace->as_string, "\n";
+        my $error = $_;
+        if ( blessed( $error ) && $error->isa( 'Kafka::Exception' ) ) {
+            warn $error->message, "\n", $error->trace->as_string, "\n";
             exit;
         } else {
-            die $_;
+            die $error;
         }
     };
 
     # Closes the connection and cleans up
+    $connection->close;
     undef $connection;
 
 =head1 DESCRIPTION
@@ -224,6 +227,9 @@ const our %RETRY_ON_ERRORS => (
 #   $ERROR_MESSAGE_SIZE_TOO_LARGE           => 1,   # 10 - Message is too big
     $ERROR_STALE_CONTROLLER_EPOCH_CODE      => 1,   # 11 - Stale Controller Epoch Code
 #   $ERROR_OFFSET_METADATA_TOO_LARGE_CODE   => 1,   # 12 - Specified metadata offset is too big
+#    $ERROR_LOAD_IN_PROGRESS_CODE            => 1,   # 14 - Still loading offsets
+    $ERROR_CONSUMER_COORDINATOR_NOT_AVAILABLE_CODE  => 1,   # 15 - Topic has not yet been created
+#    $ERROR_NOT_COORDINATOR_FOR_CONSUMER_CODE    => 1,   # 16 - Request for a consumer group that it is not a coordinator for
 );
 
 #-- constructor ----------------------------------------------------------------
@@ -564,9 +570,11 @@ sub is_server_alive {
     $self->_error( $ERROR_MISMATCH_ARGUMENT, "Unknown server '$server' (is not found in the metadata)" )
         unless exists( $io_cache->{ $server } );
 
-    my $io = $self->_connectIO( $server );
-
-    return $io->is_alive;
+    if ( my $io = $self->_connectIO( $server ) ) {
+        return $io->is_alive;
+    } else {
+        return;
+    }
 }
 
 =head3 C<is_server_connected( $server )>
@@ -1037,7 +1045,9 @@ sub _update_metadata {
         foreach my $partition_metadata ( @{ $topic_metadata->{PartitionMetadata} } ) {
             $partition = $partition_metadata->{Partition};
             last METADATA_CREATION
-                if ( $ErrorCode = $partition_metadata->{ErrorCode} ) != $ERROR_NO_ERROR;
+                if ( $ErrorCode = $partition_metadata->{ErrorCode} ) != $ERROR_NO_ERROR
+                    && $ErrorCode != $ERROR_REPLICA_NOT_AVAILABLE;
+            $ErrorCode = $ERROR_NO_ERROR;
 
             my $received_partition_data = $received_metadata->{ $TopicName }->{ $partition } = {};
             my $leader = $received_partition_data->{Leader} = $partition_metadata->{Leader};
@@ -1127,7 +1137,8 @@ sub _connectIO {
             $server_data->{error}   = undef;
         } catch {
             # NOTE: it is possible to repeat the operation here
-            $self->_on_io_error( $server_data, $_ );
+            my $error = $_;
+            $self->_on_io_error( $server_data, $error );
             return;
         };
     }
@@ -1144,7 +1155,8 @@ sub _sendIO {
         $sent = $server_data->{IO}->send( $encoded_request );
     } catch {
         # NOTE: it is possible to repeat the operation here
-        $self->_on_io_error( $server_data, $_ );
+        my $error = $_;
+        $self->_on_io_error( $server_data, $error );
     };
     return $sent;
 }
@@ -1178,7 +1190,7 @@ sub _receiveIO {
             ) if $self->debug_level;
         sleep $self->{RETRY_BACKOFF} / 1000;
     }
-    $self->_on_io_error( $server_data, $_ )
+    $self->_on_io_error( $server_data, $error )
         if $error;
 
     return $response_ref;
