@@ -6,7 +6,7 @@ Kafka::Consumer - Perl interface for Kafka consumer client.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Consumer> version 0.8010 .
+This documentation refers to C<Kafka::Consumer> version 0.8011 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.8010';
+our $VERSION = '0.8011';
 
 #-- load the modules -----------------------------------------------------------
 
@@ -128,16 +128,18 @@ if ( !$BITS64 ) { eval 'use Kafka::Int64; 1;' or die "Cannot load Kafka::Int64 :
         }
 
     } catch {
-        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
-            warn 'Error: (', $_->code, ') ',  $_->message, "\n";
+        my $error = $_;
+        if ( blessed( $error ) && $error->isa( 'Kafka::Exception' ) ) {
+            warn 'Error: (', $error->code, ') ',  $error->message, "\n";
             exit;
         } else {
-            die $_;
+            die $error;
         }
     };
 
     # Closes the consumer and cleans up
     undef $consumer;
+    $connection->close;
     undef $connection;
 
 =head1 DESCRIPTION
