@@ -6,7 +6,7 @@ Kafka::Exceptions - Perl Kafka API exception definitions.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Exceptions> version 0.8010 .
+This documentation refers to C<Kafka::Exceptions> version 0.8011 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.8010';
+our $VERSION = '0.8011';
 
 use Exporter qw(
     import
@@ -78,20 +78,22 @@ use Kafka qw(
     try {
         $connection = Kafka::Connection->new( host => 'localhost' );
     } catch {
-        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
-            if ( $_->isa( 'Kafka::Exception::Connection' ) ) {
+        my $error = $_;
+        if ( blessed( $error ) && $error->isa( 'Kafka::Exception' ) ) {
+            if ( $error->isa( 'Kafka::Exception::Connection' ) ) {
                 # Specific treatment for 'Kafka::Connection' class error
-            } elsif ( $_->isa( 'Kafka::Exception::IO' ) ) {
+            } elsif ( $error->isa( 'Kafka::Exception::IO' ) ) {
                 # Specific treatment for 'Kafka::IO' class error
             }
-            warn ref( $_ ), " error:\n", $_->message, "\n", $_->trace->as_string, "\n";
+            warn ref( $error ), " error:\n", $error->message, "\n", $error->trace->as_string, "\n";
             exit;
         } else {
-            die $_;
+            die $error;
         }
     };
 
     # Closes the connection and cleans up
+    $connection->close;
     undef $connection;
 
 =head1 DESCRIPTION
