@@ -6,7 +6,7 @@ Kafka::IO - Interface to network communication with the Apache Kafka server.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::IO> version 0.8010 .
+This documentation refers to C<Kafka::IO> version 0.8011 .
 
 =cut
 
@@ -20,7 +20,7 @@ use warnings;
 
 our $DEBUG = 0;
 
-our $VERSION = '0.8010';
+our $VERSION = '0.8011';
 
 #-- load the modules -----------------------------------------------------------
 
@@ -34,6 +34,7 @@ use Scalar::Util qw(
     dualvar
 );
 use Socket qw(
+    IPPROTO_TCP
     PF_INET
     SOCK_STREAM
     SOL_SOCKET
@@ -84,11 +85,12 @@ use Kafka::Internals qw(
     try {
         $io = Kafka::IO->new( host => 'localhost' );
     } catch {
-        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
-            warn 'Error: (', $_->code, ') ',  $_->message, "\n";
+        my $error = $_;
+        if ( blessed( $error ) && $error->isa( 'Kafka::Exception' ) ) {
+            warn 'Error: (', $error->code, ') ',  $error->message, "\n";
             exit;
         } else {
-            die $_;
+            die $error;
         }
     };
 
@@ -198,7 +200,8 @@ sub new {
     try {
         $self->_connect();
     } catch {
-        $self->_error( $ERROR_CANNOT_BIND, "->new - $_" );
+        my $error = $_;
+        $self->_error( $ERROR_CANNOT_BIND, "->new - $error" );
     };
 
     return $self;
@@ -307,10 +310,14 @@ The method verifies whether we are connected to Kafka server.
 sub is_alive {
     my ( $self ) = @_;
 
-    return unless $self->{socket};
-    return unless defined( my $packed = getsockopt( $self->{socket}, SOL_SOCKET, SO_ERROR ) );
+    my $socket = $self->{socket};
+    return unless $socket;
 
-    return !unpack( q{L}, $packed );
+    socket( my $tmp_socket, PF_INET, SOCK_STREAM, IPPROTO_TCP );
+    my $is_alive = connect( $tmp_socket, getpeername( $socket ) );
+    CORE::close( $tmp_socket );
+
+    return $is_alive;
 }
 
 #-- private attributes ---------------------------------------------------------
@@ -395,11 +402,11 @@ sub _connect {
     $_ = select( $connection ); $| = 1; select $_;
 
     # Set FD_CLOEXEC.
-    $_ = fcntl( $connection, F_GETFL, 0 ) or die "fcntl: $!\n";
-    fcntl( $connection, F_SETFL, $_ | FD_CLOEXEC ) or die "fnctl: $!\n";
+    my $flags = fcntl( $connection, F_GETFL, 0 ) or die "fcntl: $!\n";
+    fcntl( $connection, F_SETFL, $flags | FD_CLOEXEC ) or die "fnctl: $!\n";
 
-    $_ = fcntl( $connection, F_GETFL, 0 ) or die "fcntl F_GETFL: $!\n"; # 0 for error, 0e0 for 0.
-    fcntl( $connection, F_SETFL, $_ | O_NONBLOCK ) or die "fcntl F_SETFL O_NONBLOCK: $!\n"; # 0 for error, 0e0 for 0.
+    $flags = fcntl( $connection, F_GETFL, 0 ) or die "fcntl F_GETFL: $!\n"; # 0 for error, 0e0 for 0.
+    fcntl( $connection, F_SETFL, $flags | O_NONBLOCK ) or die "fcntl F_SETFL O_NONBLOCK: $!\n"; # 0 for error, 0e0 for 0.
 
     # Connect returns immediately because of O_NONBLOCK.
     connect( $connection, pack_sockaddr_in( $port, inet_aton( $ip ) ) ) || $!{EINPROGRESS} || die( "connect ${ip}:${port} (${name}): $!\n" );
@@ -408,8 +415,8 @@ sub _connect {
     $self->{_select}    = undef;
 
     # Reset O_NONBLOCK.
-    $_ = fcntl( $connection, F_GETFL, 0 ) or die "fcntl F_GETFL: $!\n";  # 0 for error, 0e0 for 0.
-    fcntl( $connection, F_SETFL, $_ & ~ O_NONBLOCK ) or die "fcntl F_SETFL not O_NONBLOCK: $!\n";  # 0 for error, 0e0 for 0.
+    $flags = fcntl( $connection, F_GETFL, 0 ) or die "fcntl F_GETFL: $!\n";  # 0 for error, 0e0 for 0.
+    fcntl( $connection, F_SETFL, $flags & ~ O_NONBLOCK ) or die "fcntl F_SETFL not O_NONBLOCK: $!\n";  # 0 for error, 0e0 for 0.
 
     # Use select() to poll for completion or error. When connect succeeds we can write.
     my $vec = q{};
@@ -505,12 +512,6 @@ sub _error {
 
 #-- Closes and cleans up -------------------------------------------------------
 
-sub DESTROY {
-    my ( $self ) = @_;
-
-    $self->close();
-}
-
 1;
 
 __END__
