@@ -7,7 +7,7 @@ protocol on 32 bit systems.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Int64> version 0.8010 .
+This documentation refers to C<Kafka::Int64> version 0.8011 .
 
 =cut
 
@@ -21,7 +21,7 @@ use bigint; # this allows integers of practially any size at the cost of signifi
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.8010';
+our $VERSION = '0.8011';
 
 use Exporter qw(
     import
@@ -81,11 +81,12 @@ use Kafka::Exceptions;
             : Kafka::Int64::intsum( $offset, 1 );
 
     } catch {
-        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
-            warn 'Error: (', $_->code, ') ',  $_->message, "\n";
+        my $error = $_;
+        if ( blessed( $error ) && $error->isa( 'Kafka::Exception' ) ) {
+            warn 'Error: (', $error->code, ') ',  $error->message, "\n";
             exit;
         } else {
-            die $_;
+            die $error;
         }
     };
 
