@@ -6,7 +6,7 @@ Kafka::Producer - Perl interface for Kafka producer client.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Producer> version 0.8010 .
+This documentation refers to C<Kafka::Producer> version 0.8011 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.8010';
+our $VERSION = '0.8011';
 
 #-- load the modules -----------------------------------------------------------
 
@@ -100,16 +100,18 @@ use Kafka::Internals qw(
         );
 
     } catch {
-        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
-            warn 'Error: (', $_->code, ') ',  $_->message, "\n";
+        my $error = $_;
+        if ( blessed( $error ) && $error->isa( 'Kafka::Exception' ) ) {
+            warn 'Error: (', $error->code, ') ',  $error->message, "\n";
             exit;
         } else {
-            die $_;
+            die $error;
         }
     };
 
     # Closes the producer and cleans up
     undef $producer;
+    $connection->close;
     undef $connection;
 
 =head1 DESCRIPTION
